#include <dbus/dbus.h>

#include <nih/macros.h>
#include <nih/alloc.h>
#include <nih/string.h>
#include <nih/logging.h>
#include <nih/error.h>

#include "tests/demarshal_code.h"

int
my_byte_demarshal (const void *parent, DBusMessage *message, uint8_t * value)
{
	DBusMessageIter iter;
	uint8_t local;

	nih_assert (message != NULL);

	nih_assert (dbus_message_iter_init (message, &iter));

	/* Demarshal a uint8_t from the message */
	if (dbus_message_iter_get_arg_type (&iter) != DBUS_TYPE_BYTE) {
		return 1;
	}

	dbus_message_iter_get_basic (&iter, &local);

	dbus_message_iter_next (&iter);

	*value = local;

	return 0;
}

int
my_boolean_demarshal (const void *parent, DBusMessage *message, int * value)
{
	DBusMessageIter iter;
	int local;

	nih_assert (message != NULL);

	nih_assert (dbus_message_iter_init (message, &iter));

	/* Demarshal a int from the message */
	if (dbus_message_iter_get_arg_type (&iter) != DBUS_TYPE_BOOLEAN) {
		return 1;
	}

	dbus_message_iter_get_basic (&iter, &local);

	dbus_message_iter_next (&iter);

	*value = local;

	return 0;
}

int
my_int16_demarshal (const void *parent, DBusMessage *message, int16_t * value)
{
	DBusMessageIter iter;
	int16_t local;

	nih_assert (message != NULL);

	nih_assert (dbus_message_iter_init (message, &iter));

	/* Demarshal a int16_t from the message */
	if (dbus_message_iter_get_arg_type (&iter) != DBUS_TYPE_INT16) {
		return 1;
	}

	dbus_message_iter_get_basic (&iter, &local);

	dbus_message_iter_next (&iter);

	*value = local;

	return 0;
}

int
my_uint16_demarshal (const void *parent, DBusMessage *message, uint16_t * value)
{
	DBusMessageIter iter;
	uint16_t local;

	nih_assert (message != NULL);

	nih_assert (dbus_message_iter_init (message, &iter));

	/* Demarshal a uint16_t from the message */
	if (dbus_message_iter_get_arg_type (&iter) != DBUS_TYPE_UINT16) {
		return 1;
	}

	dbus_message_iter_get_basic (&iter, &local);

	dbus_message_iter_next (&iter);

	*value = local;

	return 0;
}

int
my_int32_demarshal (const void *parent, DBusMessage *message, int32_t * value)
{
	DBusMessageIter iter;
	int32_t local;

	nih_assert (message != NULL);

	nih_assert (dbus_message_iter_init (message, &iter));

	/* Demarshal a int32_t from the message */
	if (dbus_message_iter_get_arg_type (&iter) != DBUS_TYPE_INT32) {
		return 1;
	}

	dbus_message_iter_get_basic (&iter, &local);

	dbus_message_iter_next (&iter);

	*value = local;

	return 0;
}

int
my_uint32_demarshal (const void *parent, DBusMessage *message, uint32_t * value)
{
	DBusMessageIter iter;
	uint32_t local;

	nih_assert (message != NULL);

	nih_assert (dbus_message_iter_init (message, &iter));

	/* Demarshal a uint32_t from the message */
	if (dbus_message_iter_get_arg_type (&iter) != DBUS_TYPE_UINT32) {
		return 1;
	}

	dbus_message_iter_get_basic (&iter, &local);

	dbus_message_iter_next (&iter);

	*value = local;

	return 0;
}

int
my_int64_demarshal (const void *parent, DBusMessage *message, int64_t * value)
{
	DBusMessageIter iter;
	int64_t local;

	nih_assert (message != NULL);

	nih_assert (dbus_message_iter_init (message, &iter));

	/* Demarshal a int64_t from the message */
	if (dbus_message_iter_get_arg_type (&iter) != DBUS_TYPE_INT64) {
		return 1;
	}

	dbus_message_iter_get_basic (&iter, &local);

	dbus_message_iter_next (&iter);

	*value = local;

	return 0;
}

int
my_uint64_demarshal (const void *parent, DBusMessage *message, uint64_t * value)
{
	DBusMessageIter iter;
	uint64_t local;

	nih_assert (message != NULL);

	nih_assert (dbus_message_iter_init (message, &iter));

	/* Demarshal a uint64_t from the message */
	if (dbus_message_iter_get_arg_type (&iter) != DBUS_TYPE_UINT64) {
		return 1;
	}

	dbus_message_iter_get_basic (&iter, &local);

	dbus_message_iter_next (&iter);

	*value = local;

	return 0;
}

int
my_double_demarshal (const void *parent, DBusMessage *message, double * value)
{
	DBusMessageIter iter;
	double local;

	nih_assert (message != NULL);

	nih_assert (dbus_message_iter_init (message, &iter));

	/* Demarshal a double from the message */
	if (dbus_message_iter_get_arg_type (&iter) != DBUS_TYPE_DOUBLE) {
		return 1;
	}

	dbus_message_iter_get_basic (&iter, &local);

	dbus_message_iter_next (&iter);

	*value = local;

	return 0;
}

int
my_string_demarshal (const void *parent, DBusMessage *message, char ** value)
{
	DBusMessageIter iter;
	const char * local_dbus;
	char * local;

	nih_assert (message != NULL);

	nih_assert (dbus_message_iter_init (message, &iter));

	/* Demarshal a char * from the message */
	if (dbus_message_iter_get_arg_type (&iter) != DBUS_TYPE_STRING) {
		return 1;
	}

	dbus_message_iter_get_basic (&iter, &local_dbus);

	local = nih_strdup (parent, local_dbus);
	if (! local) {
		return -1;
	}

	dbus_message_iter_next (&iter);

	*value = local;

	return 0;
}

int
my_object_path_demarshal (const void *parent, DBusMessage *message, char ** value)
{
	DBusMessageIter iter;
	const char * local_dbus;
	char * local;

	nih_assert (message != NULL);

	nih_assert (dbus_message_iter_init (message, &iter));

	/* Demarshal a char * from the message */
	if (dbus_message_iter_get_arg_type (&iter) != DBUS_TYPE_OBJECT_PATH) {
		return 1;
	}

	dbus_message_iter_get_basic (&iter, &local_dbus);

	local = nih_strdup (parent, local_dbus);
	if (! local) {
		return -1;
	}

	dbus_message_iter_next (&iter);

	*value = local;

	return 0;
}

int
my_signature_demarshal (const void *parent, DBusMessage *message, char ** value)
{
	DBusMessageIter iter;
	const char * local_dbus;
	char * local;

	nih_assert (message != NULL);

	nih_assert (dbus_message_iter_init (message, &iter));

	/* Demarshal a char * from the message */
	if (dbus_message_iter_get_arg_type (&iter) != DBUS_TYPE_SIGNATURE) {
		return 1;
	}

	dbus_message_iter_get_basic (&iter, &local_dbus);

	local = nih_strdup (parent, local_dbus);
	if (! local) {
		return -1;
	}

	dbus_message_iter_next (&iter);

	*value = local;

	return 0;
}

int
my_int16_array_demarshal (const void *parent, DBusMessage *message, int16_t ** value, size_t * value_len)
{
	DBusMessageIter iter;
	DBusMessageIter local_iter;
	int16_t * local;
	size_t local_len;

	nih_assert (message != NULL);

	nih_assert (dbus_message_iter_init (message, &iter));

	/* Demarshal an array from the message */
	if (dbus_message_iter_get_arg_type (&iter) != DBUS_TYPE_ARRAY) {
		return 1;
	}

	dbus_message_iter_recurse (&iter, &local_iter);

	local_len = 0;
	local = NULL;

	while (dbus_message_iter_get_arg_type (&local_iter) != DBUS_TYPE_INVALID) {
		int16_t *local_tmp;
		int16_t  local_element;

		/* Demarshal a int16_t from the message */
		if (dbus_message_iter_get_arg_type (&local_iter) != DBUS_TYPE_INT16) {
			if (local)
				nih_free (local);
			return 1;
		}

		dbus_message_iter_get_basic (&local_iter, &local_element);

		dbus_message_iter_next (&local_iter);

		if (local_len + 1 > SIZE_MAX / sizeof (int16_t)) {
			if (local)
				nih_free (local);
			return 1;
		}

		local_tmp = nih_realloc (local, parent, sizeof (int16_t) * (local_len + 1));
		if (! local_tmp) {
			if (local)
				nih_free (local);
			return -1;
		}

		local = local_tmp;
		local[local_len] = local_element;

		local_len++;
	}

	dbus_message_iter_next (&iter);

	*value = local;
	*value_len = local_len;

	return 0;
}

int
my_int16_array_array_demarshal (const void *parent, DBusMessage *message, int16_t *** value, size_t ** value_len)
{
	DBusMessageIter iter;
	DBusMessageIter local_iter;
	size_t local_size;
	int16_t ** local;
	size_t * local_len;

	nih_assert (message != NULL);

	nih_assert (dbus_message_iter_init (message, &iter));

	/* Demarshal an array from the message */
	if (dbus_message_iter_get_arg_type (&iter) != DBUS_TYPE_ARRAY) {
		return 1;
	}

	dbus_message_iter_recurse (&iter, &local_iter);

	local_size = 0;
	local = NULL;
	local_len = NULL;

	local = nih_alloc (parent, sizeof (int16_t *));
	if (! local) {
		return -1;
	}

	local[local_size] = NULL;

	while (dbus_message_iter_get_arg_type (&local_iter) != DBUS_TYPE_INVALID) {
		DBusMessageIter local_element_iter;
		int16_t **      local_tmp;
		int16_t *       local_element;
		size_t *        local_len_tmp;
		size_t          local_element_len;

		/* Demarshal an array from the message */
		if (dbus_message_iter_get_arg_type (&local_iter) != DBUS_TYPE_ARRAY) {
			if (local)
				nih_free (local);
			return 1;
		}

		dbus_message_iter_recurse (&local_iter, &local_element_iter);

		local_element_len = 0;
		local_element = NULL;

		while (dbus_message_iter_get_arg_type (&local_element_iter) != DBUS_TYPE_INVALID) {
			int16_t *local_element_tmp;
			int16_t  local_element_element;

			/* Demarshal a int16_t from the message */
			if (dbus_message_iter_get_arg_type (&local_element_iter) != DBUS_TYPE_INT16) {
				if (local_element)
					nih_free (local_element);
				if (local)
					nih_free (local);
				return 1;
			}

			dbus_message_iter_get_basic (&local_element_iter, &local_element_element);

			dbus_message_iter_next (&local_element_iter);

			if (local_element_len + 1 > SIZE_MAX / sizeof (int16_t)) {
				if (local_element)
					nih_free (local_element);
				if (local)
					nih_free (local);
				return 1;
			}

			local_element_tmp = nih_realloc (local_element, local, sizeof (int16_t) * (local_element_len + 1));
			if (! local_element_tmp) {
				if (local_element)
					nih_free (local_element);
				if (local)
					nih_free (local);
				return -1;
			}

			local_element = local_element_tmp;
			local_element[local_element_len] = local_element_element;

			local_element_len++;
		}

		dbus_message_iter_next (&local_iter);

		if (local_size + 2 > SIZE_MAX / sizeof (int16_t *)) {
			if (local)
				nih_free (local);
			return 1;
		}

		local_tmp = nih_realloc (local, parent, sizeof (int16_t *) * (local_size + 2));
		if (! local_tmp) {
			if (local)
				nih_free (local);
			return -1;
		}

		local = local_tmp;
		local[local_size] = local_element;
		local[local_size + 1] = NULL;

		if (local_size + 1 > SIZE_MAX / sizeof (size_t)) {
			if (local)
				nih_free (local);
			return 1;
		}

		local_len_tmp = nih_realloc (local_len, local, sizeof (size_t) * (local_size + 1));
		if (! local_len_tmp) {
			if (local)
				nih_free (local);
			return -1;
		}

		local_len = local_len_tmp;
		local_len[local_size] = local_element_len;

		local_size++;
	}

	dbus_message_iter_next (&iter);

	*value = local;
	*value_len = local_len;

	return 0;
}

int
my_string_array_demarshal (const void *parent, DBusMessage *message, char *** value)
{
	DBusMessageIter iter;
	DBusMessageIter local_iter;
	size_t local_size;
	char ** local;

	nih_assert (message != NULL);

	nih_assert (dbus_message_iter_init (message, &iter));

	/* Demarshal an array from the message */
	if (dbus_message_iter_get_arg_type (&iter) != DBUS_TYPE_ARRAY) {
		return 1;
	}

	dbus_message_iter_recurse (&iter, &local_iter);

	local_size = 0;
	local = NULL;

	local = nih_alloc (parent, sizeof (char *));
	if (! local) {
		return -1;
	}

	local[local_size] = NULL;

	while (dbus_message_iter_get_arg_type (&local_iter) != DBUS_TYPE_INVALID) {
		const char *local_element_dbus;
		char **     local_tmp;
		char *      local_element;

		/* Demarshal a char * from the message */
		if (dbus_message_iter_get_arg_type (&local_iter) != DBUS_TYPE_STRING) {
			if (local)
				nih_free (local);
			return 1;
		}

		dbus_message_iter_get_basic (&local_iter, &local_element_dbus);

		local_element = nih_strdup (local, local_element_dbus);
		if (! local_element) {
			if (local)
				nih_free (local);
			return -1;
		}

		dbus_message_iter_next (&local_iter);

		if (local_size + 2 > SIZE_MAX / sizeof (char *)) {
			if (local)
				nih_free (local);
			return 1;
		}

		local_tmp = nih_realloc (local, parent, sizeof (char *) * (local_size + 2));
		if (! local_tmp) {
			if (local)
				nih_free (local);
			return -1;
		}

		local = local_tmp;
		local[local_size] = local_element;
		local[local_size + 1] = NULL;

		local_size++;
	}

	dbus_message_iter_next (&iter);

	*value = local;

	return 0;
}

int
my_string_array_array_demarshal (const void *parent, DBusMessage *message, char **** value)
{
	DBusMessageIter iter;
	DBusMessageIter local_iter;
	size_t local_size;
	char *** local;

	nih_assert (message != NULL);

	nih_assert (dbus_message_iter_init (message, &iter));

	/* Demarshal an array from the message */
	if (dbus_message_iter_get_arg_type (&iter) != DBUS_TYPE_ARRAY) {
		return 1;
	}

	dbus_message_iter_recurse (&iter, &local_iter);

	local_size = 0;
	local = NULL;

	local = nih_alloc (parent, sizeof (char **));
	if (! local) {
		return -1;
	}

	local[local_size] = NULL;

	while (dbus_message_iter_get_arg_type (&local_iter) != DBUS_TYPE_INVALID) {
		DBusMessageIter local_element_iter;
		size_t          local_element_size;
		char ***        local_tmp;
		char **         local_element;

		/* Demarshal an array from the message */
		if (dbus_message_iter_get_arg_type (&local_iter) != DBUS_TYPE_ARRAY) {
			if (local)
				nih_free (local);
			return 1;
		}

		dbus_message_iter_recurse (&local_iter, &local_element_iter);

		local_element_size = 0;
		local_element = NULL;

		local_element = nih_alloc (local, sizeof (char *));
		if (! local_element) {
			if (local)
				nih_free (local);
			return -1;
		}

		local_element[local_element_size] = NULL;

		while (dbus_message_iter_get_arg_type (&local_element_iter) != DBUS_TYPE_INVALID) {
			const char *local_element_element_dbus;
			char **     local_element_tmp;
			char *      local_element_element;

			/* Demarshal a char * from the message */
			if (dbus_message_iter_get_arg_type (&local_element_iter) != DBUS_TYPE_STRING) {
				if (local_element)
					nih_free (local_element);
				if (local)
					nih_free (local);
				return 1;
			}

			dbus_message_iter_get_basic (&local_element_iter, &local_element_element_dbus);

			local_element_element = nih_strdup (local_element, local_element_element_dbus);
			if (! local_element_element) {
				if (local_element)
					nih_free (local_element);
				if (local)
					nih_free (local);
				return -1;
			}

			dbus_message_iter_next (&local_element_iter);

			if (local_element_size + 2 > SIZE_MAX / sizeof (char *)) {
				if (local_element)
					nih_free (local_element);
				if (local)
					nih_free (local);
				return 1;
			}

			local_element_tmp = nih_realloc (local_element, local, sizeof (char *) * (local_element_size + 2));
			if (! local_element_tmp) {
				if (local_element)
					nih_free (local_element);
				if (local)
					nih_free (local);
				return -1;
			}

			local_element = local_element_tmp;
			local_element[local_element_size] = local_element_element;
			local_element[local_element_size + 1] = NULL;

			local_element_size++;
		}

		dbus_message_iter_next (&local_iter);

		if (local_size + 2 > SIZE_MAX / sizeof (char **)) {
			if (local)
				nih_free (local);
			return 1;
		}

		local_tmp = nih_realloc (local, parent, sizeof (char **) * (local_size + 2));
		if (! local_tmp) {
			if (local)
				nih_free (local);
			return -1;
		}

		local = local_tmp;
		local[local_size] = local_element;
		local[local_size + 1] = NULL;

		local_size++;
	}

	dbus_message_iter_next (&iter);

	*value = local;

	return 0;
}

int
my_struct_demarshal (const void *parent, DBusMessage *message, MyStructValue ** value)
{
	DBusMessageIter iter;
	DBusMessageIter local_iter;
	const char * local_item0_dbus;
	char * local_item0;
	uint32_t local_item1;
	DBusMessageIter local_item2_iter;
	size_t local_item2_size;
	char ** local_item2;
	DBusMessageIter local_item3_iter;
	int16_t * local_item3;
	size_t local_item3_len;
	MyStructValue * local;

	nih_assert (message != NULL);

	nih_assert (dbus_message_iter_init (message, &iter));

	/* Demarshal a structure from the message */
	if (dbus_message_iter_get_arg_type (&iter) != DBUS_TYPE_STRUCT) {
		return 1;
	}

	dbus_message_iter_recurse (&iter, &local_iter);

	local = nih_new (parent, MyStructValue);
	if (! local) {
		return -1;
	}

	/* Demarshal a char * from the message */
	if (dbus_message_iter_get_arg_type (&local_iter) != DBUS_TYPE_STRING) {
		nih_free (local);
		return 1;
	}

	dbus_message_iter_get_basic (&local_iter, &local_item0_dbus);

	local_item0 = nih_strdup (local, local_item0_dbus);
	if (! local_item0) {
		nih_free (local);
		return -1;
	}

	dbus_message_iter_next (&local_iter);

	local->item0 = local_item0;

	/* Demarshal a uint32_t from the message */
	if (dbus_message_iter_get_arg_type (&local_iter) != DBUS_TYPE_UINT32) {
		nih_free (local);
		return 1;
	}

	dbus_message_iter_get_basic (&local_iter, &local_item1);

	dbus_message_iter_next (&local_iter);

	local->item1 = local_item1;

	/* Demarshal an array from the message */
	if (dbus_message_iter_get_arg_type (&local_iter) != DBUS_TYPE_ARRAY) {
		nih_free (local);
		return 1;
	}

	dbus_message_iter_recurse (&local_iter, &local_item2_iter);

	local_item2_size = 0;
	local_item2 = NULL;

	local_item2 = nih_alloc (local, sizeof (char *));
	if (! local_item2) {
		nih_free (local);
		return -1;
	}

	local_item2[local_item2_size] = NULL;

	while (dbus_message_iter_get_arg_type (&local_item2_iter) != DBUS_TYPE_INVALID) {
		const char *local_item2_element_dbus;
		char **     local_item2_tmp;
		char *      local_item2_element;

		/* Demarshal a char * from the message */
		if (dbus_message_iter_get_arg_type (&local_item2_iter) != DBUS_TYPE_STRING) {
			if (local_item2)
				nih_free (local_item2);
			nih_free (local);
			return 1;
		}

		dbus_message_iter_get_basic (&local_item2_iter, &local_item2_element_dbus);

		local_item2_element = nih_strdup (local_item2, local_item2_element_dbus);
		if (! local_item2_element) {
			if (local_item2)
				nih_free (local_item2);
			nih_free (local);
			return -1;
		}

		dbus_message_iter_next (&local_item2_iter);

		if (local_item2_size + 2 > SIZE_MAX / sizeof (char *)) {
			if (local_item2)
				nih_free (local_item2);
			nih_free (local);
			return 1;
		}

		local_item2_tmp = nih_realloc (local_item2, local, sizeof (char *) * (local_item2_size + 2));
		if (! local_item2_tmp) {
			if (local_item2)
				nih_free (local_item2);
			nih_free (local);
			return -1;
		}

		local_item2 = local_item2_tmp;
		local_item2[local_item2_size] = local_item2_element;
		local_item2[local_item2_size + 1] = NULL;

		local_item2_size++;
	}

	dbus_message_iter_next (&local_iter);

	local->item2 = local_item2;

	/* Demarshal an array from the message */
	if (dbus_message_iter_get_arg_type (&local_iter) != DBUS_TYPE_ARRAY) {
		nih_free (local);
		return 1;
	}

	dbus_message_iter_recurse (&local_iter, &local_item3_iter);

	local_item3_len = 0;
	local_item3 = NULL;

	while (dbus_message_iter_get_arg_type (&local_item3_iter) != DBUS_TYPE_INVALID) {
		int16_t *local_item3_tmp;
		int16_t  local_item3_element;

		/* Demarshal a int16_t from the message */
		if (dbus_message_iter_get_arg_type (&local_item3_iter) != DBUS_TYPE_INT16) {
			if (local_item3)
				nih_free (local_item3);
			nih_free (local);
			return 1;
		}

		dbus_message_iter_get_basic (&local_item3_iter, &local_item3_element);

		dbus_message_iter_next (&local_item3_iter);

		if (local_item3_len + 1 > SIZE_MAX / sizeof (int16_t)) {
			if (local_item3)
				nih_free (local_item3);
			nih_free (local);
			return 1;
		}

		local_item3_tmp = nih_realloc (local_item3, local, sizeof (int16_t) * (local_item3_len + 1));
		if (! local_item3_tmp) {
			if (local_item3)
				nih_free (local_item3);
			nih_free (local);
			return -1;
		}

		local_item3 = local_item3_tmp;
		local_item3[local_item3_len] = local_item3_element;

		local_item3_len++;
	}

	dbus_message_iter_next (&local_iter);

	local->item3 = local_item3;
	local->item3_len = local_item3_len;

	if (dbus_message_iter_get_arg_type (&local_iter) != DBUS_TYPE_INVALID) {
		nih_free (local);
		return 1;
	}

	dbus_message_iter_next (&iter);

	*value = local;

	return 0;
}

int
my_struct_array_demarshal (const void *parent, DBusMessage *message, MyStructArrayValueElement *** value)
{
	DBusMessageIter iter;
	DBusMessageIter local_iter;
	size_t local_size;
	MyStructArrayValueElement ** local;

	nih_assert (message != NULL);

	nih_assert (dbus_message_iter_init (message, &iter));

	/* Demarshal an array from the message */
	if (dbus_message_iter_get_arg_type (&iter) != DBUS_TYPE_ARRAY) {
		return 1;
	}

	dbus_message_iter_recurse (&iter, &local_iter);

	local_size = 0;
	local = NULL;

	local = nih_alloc (parent, sizeof (MyStructArrayValueElement *));
	if (! local) {
		return -1;
	}

	local[local_size] = NULL;

	while (dbus_message_iter_get_arg_type (&local_iter) != DBUS_TYPE_INVALID) {
		DBusMessageIter             local_element_iter;
		const char *                local_element_item0_dbus;
		char *                      local_element_item0;
		uint32_t                    local_element_item1;
		MyStructArrayValueElement **local_tmp;
		MyStructArrayValueElement * local_element;

		/* Demarshal a structure from the message */
		if (dbus_message_iter_get_arg_type (&local_iter) != DBUS_TYPE_STRUCT) {
			if (local)
				nih_free (local);
			return 1;
		}

		dbus_message_iter_recurse (&local_iter, &local_element_iter);

		local_element = nih_new (local, MyStructArrayValueElement);
		if (! local_element) {
			if (local)
				nih_free (local);
			return -1;
		}

		/* Demarshal a char * from the message */
		if (dbus_message_iter_get_arg_type (&local_element_iter) != DBUS_TYPE_STRING) {
			nih_free (local_element);
			if (local)
				nih_free (local);
			return 1;
		}

		dbus_message_iter_get_basic (&local_element_iter, &local_element_item0_dbus);

		local_element_item0 = nih_strdup (local_element, local_element_item0_dbus);
		if (! local_element_item0) {
			nih_free (local_element);
			if (local)
				nih_free (local);
			return -1;
		}

		dbus_message_iter_next (&local_element_iter);

		local_element->item0 = local_element_item0;

		/* Demarshal a uint32_t from the message */
		if (dbus_message_iter_get_arg_type (&local_element_iter) != DBUS_TYPE_UINT32) {
			nih_free (local_element);
			if (local)
				nih_free (local);
			return 1;
		}

		dbus_message_iter_get_basic (&local_element_iter, &local_element_item1);

		dbus_message_iter_next (&local_element_iter);

		local_element->item1 = local_element_item1;

		if (dbus_message_iter_get_arg_type (&local_element_iter) != DBUS_TYPE_INVALID) {
			nih_free (local_element);
			if (local)
				nih_free (local);
			return 1;
		}

		dbus_message_iter_next (&local_iter);

		if (local_size + 2 > SIZE_MAX / sizeof (MyStructArrayValueElement *)) {
			if (local)
				nih_free (local);
			return 1;
		}

		local_tmp = nih_realloc (local, parent, sizeof (MyStructArrayValueElement *) * (local_size + 2));
		if (! local_tmp) {
			if (local)
				nih_free (local);
			return -1;
		}

		local = local_tmp;
		local[local_size] = local_element;
		local[local_size + 1] = NULL;

		local_size++;
	}

	dbus_message_iter_next (&iter);

	*value = local;

	return 0;
}

int
my_dict_entry_array_demarshal (const void *parent, DBusMessage *message, MyDictEntryArrayValueElement *** value)
{
	DBusMessageIter iter;
	DBusMessageIter local_iter;
	size_t local_size;
	MyDictEntryArrayValueElement ** local;

	nih_assert (message != NULL);

	nih_assert (dbus_message_iter_init (message, &iter));

	/* Demarshal an array from the message */
	if (dbus_message_iter_get_arg_type (&iter) != DBUS_TYPE_ARRAY) {
		return 1;
	}

	dbus_message_iter_recurse (&iter, &local_iter);

	local_size = 0;
	local = NULL;

	local = nih_alloc (parent, sizeof (MyDictEntryArrayValueElement *));
	if (! local) {
		return -1;
	}

	local[local_size] = NULL;

	while (dbus_message_iter_get_arg_type (&local_iter) != DBUS_TYPE_INVALID) {
		DBusMessageIter                local_element_iter;
		const char *                   local_element_item0_dbus;
		char *                         local_element_item0;
		uint32_t                       local_element_item1;
		MyDictEntryArrayValueElement **local_tmp;
		MyDictEntryArrayValueElement * local_element;

		/* Demarshal a structure from the message */
		if (dbus_message_iter_get_arg_type (&local_iter) != DBUS_TYPE_DICT_ENTRY) {
			if (local)
				nih_free (local);
			return 1;
		}

		dbus_message_iter_recurse (&local_iter, &local_element_iter);

		local_element = nih_new (local, MyDictEntryArrayValueElement);
		if (! local_element) {
			if (local)
				nih_free (local);
			return -1;
		}

		/* Demarshal a char * from the message */
		if (dbus_message_iter_get_arg_type (&local_element_iter) != DBUS_TYPE_STRING) {
			nih_free (local_element);
			if (local)
				nih_free (local);
			return 1;
		}

		dbus_message_iter_get_basic (&local_element_iter, &local_element_item0_dbus);

		local_element_item0 = nih_strdup (local_element, local_element_item0_dbus);
		if (! local_element_item0) {
			nih_free (local_element);
			if (local)
				nih_free (local);
			return -1;
		}

		dbus_message_iter_next (&local_element_iter);

		local_element->item0 = local_element_item0;

		/* Demarshal a uint32_t from the message */
		if (dbus_message_iter_get_arg_type (&local_element_iter) != DBUS_TYPE_UINT32) {
			nih_free (local_element);
			if (local)
				nih_free (local);
			return 1;
		}

		dbus_message_iter_get_basic (&local_element_iter, &local_element_item1);

		dbus_message_iter_next (&local_element_iter);

		local_element->item1 = local_element_item1;

		if (dbus_message_iter_get_arg_type (&local_element_iter) != DBUS_TYPE_INVALID) {
			nih_free (local_element);
			if (local)
				nih_free (local);
			return 1;
		}

		dbus_message_iter_next (&local_iter);

		if (local_size + 2 > SIZE_MAX / sizeof (MyDictEntryArrayValueElement *)) {
			if (local)
				nih_free (local);
			return 1;
		}

		local_tmp = nih_realloc (local, parent, sizeof (MyDictEntryArrayValueElement *) * (local_size + 2));
		if (! local_tmp) {
			if (local)
				nih_free (local);
			return -1;
		}

		local = local_tmp;
		local[local_size] = local_element;
		local[local_size + 1] = NULL;

		local_size++;
	}

	dbus_message_iter_next (&iter);

	*value = local;

	return 0;
}

