#include <dbus/dbus.h>

#include <nih/macros.h>
#include <nih/alloc.h>
#include <nih/string.h>
#include <nih/logging.h>
#include <nih/error.h>

#include "tests/marshal_code.h"

int
my_byte_marshal (DBusMessage *message, uint8_t value)
{
	DBusMessageIter iter;

	nih_assert (message != NULL);

	dbus_message_iter_init_append (message, &iter);

	/* Marshal a uint8_t onto the message */
	if (! dbus_message_iter_append_basic (&iter, DBUS_TYPE_BYTE, &value)) {
		return -1;
	}

	return 0;
}

int
my_boolean_marshal (DBusMessage *message, int value)
{
	DBusMessageIter iter;

	nih_assert (message != NULL);

	dbus_message_iter_init_append (message, &iter);

	/* Marshal a int onto the message */
	if (! dbus_message_iter_append_basic (&iter, DBUS_TYPE_BOOLEAN, &value)) {
		return -1;
	}

	return 0;
}

int
my_int16_marshal (DBusMessage *message, int16_t value)
{
	DBusMessageIter iter;

	nih_assert (message != NULL);

	dbus_message_iter_init_append (message, &iter);

	/* Marshal a int16_t onto the message */
	if (! dbus_message_iter_append_basic (&iter, DBUS_TYPE_INT16, &value)) {
		return -1;
	}

	return 0;
}

int
my_uint16_marshal (DBusMessage *message, uint16_t value)
{
	DBusMessageIter iter;

	nih_assert (message != NULL);

	dbus_message_iter_init_append (message, &iter);

	/* Marshal a uint16_t onto the message */
	if (! dbus_message_iter_append_basic (&iter, DBUS_TYPE_UINT16, &value)) {
		return -1;
	}

	return 0;
}

int
my_int32_marshal (DBusMessage *message, int32_t value)
{
	DBusMessageIter iter;

	nih_assert (message != NULL);

	dbus_message_iter_init_append (message, &iter);

	/* Marshal a int32_t onto the message */
	if (! dbus_message_iter_append_basic (&iter, DBUS_TYPE_INT32, &value)) {
		return -1;
	}

	return 0;
}

int
my_uint32_marshal (DBusMessage *message, uint32_t value)
{
	DBusMessageIter iter;

	nih_assert (message != NULL);

	dbus_message_iter_init_append (message, &iter);

	/* Marshal a uint32_t onto the message */
	if (! dbus_message_iter_append_basic (&iter, DBUS_TYPE_UINT32, &value)) {
		return -1;
	}

	return 0;
}

int
my_int64_marshal (DBusMessage *message, int64_t value)
{
	DBusMessageIter iter;

	nih_assert (message != NULL);

	dbus_message_iter_init_append (message, &iter);

	/* Marshal a int64_t onto the message */
	if (! dbus_message_iter_append_basic (&iter, DBUS_TYPE_INT64, &value)) {
		return -1;
	}

	return 0;
}

int
my_uint64_marshal (DBusMessage *message, uint64_t value)
{
	DBusMessageIter iter;

	nih_assert (message != NULL);

	dbus_message_iter_init_append (message, &iter);

	/* Marshal a uint64_t onto the message */
	if (! dbus_message_iter_append_basic (&iter, DBUS_TYPE_UINT64, &value)) {
		return -1;
	}

	return 0;
}

int
my_double_marshal (DBusMessage *message, double value)
{
	DBusMessageIter iter;

	nih_assert (message != NULL);

	dbus_message_iter_init_append (message, &iter);

	/* Marshal a double onto the message */
	if (! dbus_message_iter_append_basic (&iter, DBUS_TYPE_DOUBLE, &value)) {
		return -1;
	}

	return 0;
}

int
my_string_marshal (DBusMessage *message, const char * value)
{
	DBusMessageIter iter;

	nih_assert (message != NULL);

	dbus_message_iter_init_append (message, &iter);

	/* Marshal a char * onto the message */
	if (! dbus_message_iter_append_basic (&iter, DBUS_TYPE_STRING, &value)) {
		return -1;
	}

	return 0;
}

int
my_object_path_marshal (DBusMessage *message, const char * value)
{
	DBusMessageIter iter;

	nih_assert (message != NULL);

	dbus_message_iter_init_append (message, &iter);

	/* Marshal a char * onto the message */
	if (! dbus_message_iter_append_basic (&iter, DBUS_TYPE_OBJECT_PATH, &value)) {
		return -1;
	}

	return 0;
}

int
my_signature_marshal (DBusMessage *message, const char * value)
{
	DBusMessageIter iter;

	nih_assert (message != NULL);

	dbus_message_iter_init_append (message, &iter);

	/* Marshal a char * onto the message */
	if (! dbus_message_iter_append_basic (&iter, DBUS_TYPE_SIGNATURE, &value)) {
		return -1;
	}

	return 0;
}

int
my_int16_array_marshal (DBusMessage *message, const int16_t * value, size_t value_len)
{
	DBusMessageIter iter;
	DBusMessageIter value_iter;

	nih_assert (message != NULL);

	dbus_message_iter_init_append (message, &iter);

	/* Marshal an array onto the message */
	if (! dbus_message_iter_open_container (&iter, DBUS_TYPE_ARRAY, "n", &value_iter)) {
		return -1;
	}

	for (size_t value_i = 0; value_i < value_len; value_i++) {
		int16_t value_element;

		value_element = value[value_i];

		/* Marshal a int16_t onto the message */
		if (! dbus_message_iter_append_basic (&value_iter, DBUS_TYPE_INT16, &value_element)) {
			dbus_message_iter_abandon_container (&iter, &value_iter);
			return -1;
		}
	}

	if (! dbus_message_iter_close_container (&iter, &value_iter)) {
		return -1;
	}

	return 0;
}

int
my_int16_array_array_marshal (DBusMessage *message, int16_t * const * value, const size_t * value_len)
{
	DBusMessageIter iter;
	DBusMessageIter value_iter;

	nih_assert (message != NULL);

	dbus_message_iter_init_append (message, &iter);

	/* Marshal an array onto the message */
	if (! dbus_message_iter_open_container (&iter, DBUS_TYPE_ARRAY, "an", &value_iter)) {
		return -1;
	}

	for (size_t value_i = 0; value[value_i]; value_i++) {
		DBusMessageIter value_element_iter;
		const int16_t * value_element;
		size_t          value_element_len;

		value_element = value[value_i];
		value_element_len = value_len[value_i];

		/* Marshal an array onto the message */
		if (! dbus_message_iter_open_container (&value_iter, DBUS_TYPE_ARRAY, "n", &value_element_iter)) {
			dbus_message_iter_abandon_container (&iter, &value_iter);
			return -1;
		}

		for (size_t value_element_i = 0; value_element_i < value_element_len; value_element_i++) {
			int16_t value_element_element;

			value_element_element = value_element[value_element_i];

			/* Marshal a int16_t onto the message */
			if (! dbus_message_iter_append_basic (&value_element_iter, DBUS_TYPE_INT16, &value_element_element)) {
				dbus_message_iter_abandon_container (&value_iter, &value_element_iter);
				dbus_message_iter_abandon_container (&iter, &value_iter);
				return -1;
			}
		}

		if (! dbus_message_iter_close_container (&value_iter, &value_element_iter)) {
			dbus_message_iter_abandon_container (&iter, &value_iter);
			return -1;
		}
	}

	if (! dbus_message_iter_close_container (&iter, &value_iter)) {
		return -1;
	}

	return 0;
}

int
my_string_array_marshal (DBusMessage *message, char * const * value)
{
	DBusMessageIter iter;
	DBusMessageIter value_iter;

	nih_assert (message != NULL);

	dbus_message_iter_init_append (message, &iter);

	/* Marshal an array onto the message */
	if (! dbus_message_iter_open_container (&iter, DBUS_TYPE_ARRAY, "s", &value_iter)) {
		return -1;
	}

	for (size_t value_i = 0; value[value_i]; value_i++) {
		const char *value_element;

		value_element = value[value_i];

		/* Marshal a char * onto the message */
		if (! dbus_message_iter_append_basic (&value_iter, DBUS_TYPE_STRING, &value_element)) {
			dbus_message_iter_abandon_container (&iter, &value_iter);
			return -1;
		}
	}

	if (! dbus_message_iter_close_container (&iter, &value_iter)) {
		return -1;
	}

	return 0;
}

int
my_string_array_array_marshal (DBusMessage *message, char ** const * value)
{
	DBusMessageIter iter;
	DBusMessageIter value_iter;

	nih_assert (message != NULL);

	dbus_message_iter_init_append (message, &iter);

	/* Marshal an array onto the message */
	if (! dbus_message_iter_open_container (&iter, DBUS_TYPE_ARRAY, "as", &value_iter)) {
		return -1;
	}

	for (size_t value_i = 0; value[value_i]; value_i++) {
		DBusMessageIter value_element_iter;
		char * const *  value_element;

		value_element = value[value_i];

		/* Marshal an array onto the message */
		if (! dbus_message_iter_open_container (&value_iter, DBUS_TYPE_ARRAY, "s", &value_element_iter)) {
			dbus_message_iter_abandon_container (&iter, &value_iter);
			return -1;
		}

		for (size_t value_element_i = 0; value_element[value_element_i]; value_element_i++) {
			const char *value_element_element;

			value_element_element = value_element[value_element_i];

			/* Marshal a char * onto the message */
			if (! dbus_message_iter_append_basic (&value_element_iter, DBUS_TYPE_STRING, &value_element_element)) {
				dbus_message_iter_abandon_container (&value_iter, &value_element_iter);
				dbus_message_iter_abandon_container (&iter, &value_iter);
				return -1;
			}
		}

		if (! dbus_message_iter_close_container (&value_iter, &value_element_iter)) {
			dbus_message_iter_abandon_container (&iter, &value_iter);
			return -1;
		}
	}

	if (! dbus_message_iter_close_container (&iter, &value_iter)) {
		return -1;
	}

	return 0;
}

int
my_struct_marshal (DBusMessage *message, const MyStructValue * value)
{
	DBusMessageIter iter;
	DBusMessageIter value_iter;
	const char * value_item0;
	uint32_t value_item1;
	DBusMessageIter value_item2_iter;
	char * const * value_item2;
	DBusMessageIter value_item3_iter;
	const int16_t * value_item3;
	size_t value_item3_len;

	nih_assert (message != NULL);

	dbus_message_iter_init_append (message, &iter);

	/* Marshal a structure onto the message */
	if (! dbus_message_iter_open_container (&iter, DBUS_TYPE_STRUCT, NULL, &value_iter)) {
		return -1;
	}

	value_item0 = value->item0;

	/* Marshal a char * onto the message */
	if (! dbus_message_iter_append_basic (&value_iter, DBUS_TYPE_STRING, &value_item0)) {
		dbus_message_iter_abandon_container (&iter, &value_iter);
		return -1;
	}

	value_item1 = value->item1;

	/* Marshal a uint32_t onto the message */
	if (! dbus_message_iter_append_basic (&value_iter, DBUS_TYPE_UINT32, &value_item1)) {
		dbus_message_iter_abandon_container (&iter, &value_iter);
		return -1;
	}

	value_item2 = value->item2;

	/* Marshal an array onto the message */
	if (! dbus_message_iter_open_container (&value_iter, DBUS_TYPE_ARRAY, "s", &value_item2_iter)) {
		dbus_message_iter_abandon_container (&iter, &value_iter);
		return -1;
	}

	for (size_t value_item2_i = 0; value_item2[value_item2_i]; value_item2_i++) {
		const char *value_item2_element;

		value_item2_element = value_item2[value_item2_i];

		/* Marshal a char * onto the message */
		if (! dbus_message_iter_append_basic (&value_item2_iter, DBUS_TYPE_STRING, &value_item2_element)) {
			dbus_message_iter_abandon_container (&value_iter, &value_item2_iter);
			dbus_message_iter_abandon_container (&iter, &value_iter);
			return -1;
		}
	}

	if (! dbus_message_iter_close_container (&value_iter, &value_item2_iter)) {
		dbus_message_iter_abandon_container (&iter, &value_iter);
		return -1;
	}

	value_item3 = value->item3;
	value_item3_len = value->item3_len;

	/* Marshal an array onto the message */
	if (! dbus_message_iter_open_container (&value_iter, DBUS_TYPE_ARRAY, "n", &value_item3_iter)) {
		dbus_message_iter_abandon_container (&iter, &value_iter);
		return -1;
	}

	for (size_t value_item3_i = 0; value_item3_i < value_item3_len; value_item3_i++) {
		int16_t value_item3_element;

		value_item3_element = value_item3[value_item3_i];

		/* Marshal a int16_t onto the message */
		if (! dbus_message_iter_append_basic (&value_item3_iter, DBUS_TYPE_INT16, &value_item3_element)) {
			dbus_message_iter_abandon_container (&value_iter, &value_item3_iter);
			dbus_message_iter_abandon_container (&iter, &value_iter);
			return -1;
		}
	}

	if (! dbus_message_iter_close_container (&value_iter, &value_item3_iter)) {
		dbus_message_iter_abandon_container (&iter, &value_iter);
		return -1;
	}

	if (! dbus_message_iter_close_container (&iter, &value_iter)) {
		return -1;
	}

	return 0;
}

int
my_struct_array_marshal (DBusMessage *message, MyStructArrayValueElement * const * value)
{
	DBusMessageIter iter;
	DBusMessageIter value_iter;

	nih_assert (message != NULL);

	dbus_message_iter_init_append (message, &iter);

	/* Marshal an array onto the message */
	if (! dbus_message_iter_open_container (&iter, DBUS_TYPE_ARRAY, "(su)", &value_iter)) {
		return -1;
	}

	for (size_t value_i = 0; value[value_i]; value_i++) {
		DBusMessageIter                  value_element_iter;
		const char *                     value_element_item0;
		uint32_t                         value_element_item1;
		const MyStructArrayValueElement *value_element;

		value_element = value[value_i];

		/* Marshal a structure onto the message */
		if (! dbus_message_iter_open_container (&value_iter, DBUS_TYPE_STRUCT, NULL, &value_element_iter)) {
			dbus_message_iter_abandon_container (&iter, &value_iter);
			return -1;
		}

		value_element_item0 = value_element->item0;

		/* Marshal a char * onto the message */
		if (! dbus_message_iter_append_basic (&value_element_iter, DBUS_TYPE_STRING, &value_element_item0)) {
			dbus_message_iter_abandon_container (&value_iter, &value_element_iter);
			dbus_message_iter_abandon_container (&iter, &value_iter);
			return -1;
		}

		value_element_item1 = value_element->item1;

		/* Marshal a uint32_t onto the message */
		if (! dbus_message_iter_append_basic (&value_element_iter, DBUS_TYPE_UINT32, &value_element_item1)) {
			dbus_message_iter_abandon_container (&value_iter, &value_element_iter);
			dbus_message_iter_abandon_container (&iter, &value_iter);
			return -1;
		}

		if (! dbus_message_iter_close_container (&value_iter, &value_element_iter)) {
			dbus_message_iter_abandon_container (&iter, &value_iter);
			return -1;
		}
	}

	if (! dbus_message_iter_close_container (&iter, &value_iter)) {
		return -1;
	}

	return 0;
}

int
my_dict_entry_array_marshal (DBusMessage *message, MyDictEntryArrayValueElement * const * value)
{
	DBusMessageIter iter;
	DBusMessageIter value_iter;

	nih_assert (message != NULL);

	dbus_message_iter_init_append (message, &iter);

	/* Marshal an array onto the message */
	if (! dbus_message_iter_open_container (&iter, DBUS_TYPE_ARRAY, "{su}", &value_iter)) {
		return -1;
	}

	for (size_t value_i = 0; value[value_i]; value_i++) {
		DBusMessageIter                     value_element_iter;
		const char *                        value_element_item0;
		uint32_t                            value_element_item1;
		const MyDictEntryArrayValueElement *value_element;

		value_element = value[value_i];

		/* Marshal a structure onto the message */
		if (! dbus_message_iter_open_container (&value_iter, DBUS_TYPE_DICT_ENTRY, NULL, &value_element_iter)) {
			dbus_message_iter_abandon_container (&iter, &value_iter);
			return -1;
		}

		value_element_item0 = value_element->item0;

		/* Marshal a char * onto the message */
		if (! dbus_message_iter_append_basic (&value_element_iter, DBUS_TYPE_STRING, &value_element_item0)) {
			dbus_message_iter_abandon_container (&value_iter, &value_element_iter);
			dbus_message_iter_abandon_container (&iter, &value_iter);
			return -1;
		}

		value_element_item1 = value_element->item1;

		/* Marshal a uint32_t onto the message */
		if (! dbus_message_iter_append_basic (&value_element_iter, DBUS_TYPE_UINT32, &value_element_item1)) {
			dbus_message_iter_abandon_container (&value_iter, &value_element_iter);
			dbus_message_iter_abandon_container (&iter, &value_iter);
			return -1;
		}

		if (! dbus_message_iter_close_container (&value_iter, &value_element_iter)) {
			dbus_message_iter_abandon_container (&iter, &value_iter);
			return -1;
		}
	}

	if (! dbus_message_iter_close_container (&iter, &value_iter)) {
		return -1;
	}

	return 0;
}

